/*  VER 161   TAB P   $Id: main.c,v 1.32 1997/01/02 10:38:48 src Exp $
 *
 *  an NNTP news exchange client
 *
 *  copyright 1996 Egil Kvaleberg, egilk@sn.no
 *  Husebybakken 14A, N-0379 Oslo, Norway	  
 *
 *  this program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  this program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  see the
 *  GNU General Public License for more details.
 *
 *  you should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "common.h"
#include "proto.h"

#include "getopt.h"

static void 
pickargs(int argc,char *argv[],char *envp[]);
static void 
stringarg(char *arg,char **where, char option);

char *newline = "\r\n";

/*
 *  main 
 */
int 
main(int argc, char *argv[], char *envp[])
{
    char *p,*q;
    int ok;

    /* initialize and set the name of the program */
    for (q=argv[0]; (p=strchr(q,'/')); ) q = p+1;
    stringarg(q,&pname,0);
    begintitle(argc,argv,envp);

    /* plus some command line defaults */
    nopull_opt = 0;
    inn_flag = INN_MODE;
    minspool = MINSPOOL;

    /* there may be an environment variable */
    spoolname = getenv("NNTPSERVER");

    /* parse the args */
    pickargs(argc,argv,envp);

    /* defaults */
    if (!spooldir) stringarg(SPOOL,&spooldir,0);

    /* do what we were told */
    ok = doit();

    /* remove locks */
    history_done();
    unlock();

    return ok;
}

/*
 *  pick arguments from command line
 *  NOTE: the actual command line will be cleared after this
 */
static void 
pickargs(int argc,char *argv[],char *envp[])
{
    int c;
    char *p;
    char name[PATH_MAX+PATH_MAX];
	
    int digit_optind = 0;

    for (;;) {
	int option_index = 0;
	static struct option long_options[] = {
	   /* char *name, int has_arg, int *flag, int val */
	    {"auth",	    1, 0, 'a'},
	    {"size",	    1, 0, 'b'},
	    {"cnews",	    0, 0, 'c'},
	    {"debug",	    0, 0, 'd'},
	    {"end",	    1, 0, 'e'}, 
	    {"posted",	    1, 0, 'f'}, 
	    {"nofetch",     0, 0, 'g'},
	    {"history",     1, 0, 'h'},
	    {"inn",	    0, 0, 'i'},
	    {"keeppath",    0, 0, 'k'},
	    {"log",	    1, 0, 'l'}, 
	    {"nomsgid",     0, 0, 'm'},
	    {"noaction",    0, 0, 'n'},
	    {"keepold",     0, 0, 'o'},
	    {"nopost",	    0, 0, 'p'},
	    {"reader",	    0, 0, 'r'},
	    {"spool",	    1, 0, 's'},
	    {"timeout",     1, 0, 't'},
	    {"noforce",     0, 0, 'u'},
	    {"version",     0, 0, 'v'},
	    {"chat",	    1, 0, 'w'},
	    {"exec",	    1, 0, 'x'},
	    {"connect",     1, 0, 'y'},
	    {"sync",	    0, 0, 'z'},
	    {"active",	    1, 0, 'A'},
	    {"batch",	    1, 0, 'B'},
	    {"ctlinnd",     1, 0, 'C'},
	    {"continue",    0, 0, 'E'},
	    {"home",	    1, 0, 'H'},
	    {"incoming",    1, 0, 'I'},
	    {"newline",     0, 0, 'L'},
	    {"newsfeeds",   1, 0, 'N'}, /* BUG: */
	    {"sys",	    1, 0, 'S'}, /* BUG: */
	    {"reset",	    0, 0, 'R'},
	    {"togo",	    1, 0, 'T'},
	    {"window",	    1, 0, 'W'},
	    {"syncnew",     0, 0, 'Z'},
	    {0, 0, 0, 0}
	};

	c = getopt_long(argc,argv,
		    "a:b:cde:f:gh:ikl:mnoprs:t:uvw:x:y:zA:B:H:I:LN:RT:W:Z",
		    long_options, &option_index);
	if (c == EOF) break;

	switch (c) {
	case 'a':			/* do an authinfo */
	    if (ai_username) goto usage;
	    get_authinfo(optarg);
	    break;
	case 'b':			/* min spool size in bytes */
	    minspool = atoi(optarg);		       
	    break;
	case 'c':			/* C News mode */
	    inn_flag=0;
	    break;
	case 'd':			/* debugging on */
	    debug_flag++;
	    break;
	case 'e':			/* end at tag */
	    if (end_tag) goto usage;
	    end_tag = strcpy(malloc(sizeof(optarg)+1),optarg);
	    break;
	case 'f':			/* folder */
	    stringarg(optarg,&folder,c);
	    break;
	case 'h':			/* history file */
	    stringarg(optarg,&history,c);
	    break;
	case 'g':			/* don't get news */
	    ++nopull_opt;
	    break;
	case 'i':			/* INN mode */
	    inn_flag=1;
	    break;
	case 'k':			/* keep "Path" */
	    keep_path_flag++;
	    break;
	case 'l':			/* logfile */
	    stringarg(optarg,&logfile,c);
	    break;
	case 'n':			/* do nothing */
	    noaction_flag++;
	    break;
	case 'm':			/* skip message ID */
	    nomsgid_flag++;
	    break;
	case 'o':			/* keep .old */
	    keep_old_flag++;
	    break;
	case 'p':			/* no posting */
	    nopost_opt++;
	    break;
	case 'r':			/* do a 'MODE READER' */
	    mode_reader_flag++;
	    break;
	case 's':			/* spool */
	    stringarg(optarg,&spooldir,c);
	    break;
	case 't':			/* timeout in seconds */
	    timeout = atoi(optarg);		       
	    break;
	case 'u':			/* no unlock by force */
	    noforce_flag++;
	    break;
	case 'w':			/* -w chat */
	    stringarg(optarg,&chat_file,c);
	    break;
	case 'v':			/* version */
	    printf("%s version %s\n", pname, VERSION);
	    unlock_exit(0); /* that's ok */
	    break;
	case 'x':			/* -x connect script */
	    stringarg(optarg,&connect_exec,c);
	    break;
	case 'y':			/* -y connect via */
	    stringarg(optarg,&via_exec,c);
	    break;
	case 'z':			/* zero incoming */
	    if (reset_flag || zapnew_flag) goto usage;
	    zap_flag++;
	    break;
	case 'A':			/* active file */
	    stringarg(optarg,&active,c);
	    break;
	case 'B':			/* batch directory */
	    stringarg(optarg,&batch,c);
	    break;
	case 'C':			/* ctlinnd program */
	    stringarg(optarg,&ctlpgm,c);
	    break;
	case 'E':			/* continue, ignoring some errors */
	    ++conterr;
	    break;
	case 'H':			/* news home directory */
	    stringarg(optarg,&newshome,c);
	    break;
	case 'I':			/* incoming directory */
	    stringarg(optarg,&incoming,c);
	    break;
	case 'L':			/* newline */
	    newline = "\n";
	    break;
	case 'N':			/* newsfeeds file */
	case 'S':			/* sys file */
	    stringarg(optarg,&newsfeeds,c);
	    break;
	case 'R':			/* reset incoming */
	    if (zap_flag || zapnew_flag) goto usage;
	    reset_flag++;
	    break;
	case 'T':			/* togo file */
	    stringarg(optarg,&togo,c);
	    break;
	case 'W':			/* window size */
	    window = atoi(optarg);		       
	    break;
	case 'Z':			/* zero new groups */
	    if (reset_flag || zap_flag) goto usage;
	    zapnew_flag++;
	    break;
	default:
	  usage:
	    /* BUG: have a more complete list! */
	    fprintf(stderr,"usage: %s spool [[server] port]\n", pname);
	    unlock_exit(2);
	}
    }
    
    /* get spool name */
    if (optind < argc) {
	if (argv[optind][0]) {
	    stringarg(argv[optind],&spoolname,0);
	}
	++optind;
    } 
    if (!spoolname || !spoolname[0]) {
	fprintf(stderr, "spool name missing\n");
	goto usage;
    }

    /* get server name */
    if (optind < argc) {
	if (argv[optind][0]) {
	    stringarg(argv[optind],&hostname,0);
	}
	++optind;
    }
    if (!hostname || !hostname[0]) hostname = spoolname;
    
    /* get port name */
    if (optind < argc) {
	if (argv[optind][0]) {
	    stringarg(argv[optind],&hostport,0);
	}
	++optind;
    } 
    if (!hostport || !hostport[0]) hostport = "nntp";

    if (optind < argc) {
	fprintf(stderr, "excess arguments\n");
	goto usage;
    }

    /* set the program title for "ps" */
    sprintf(name,"%s(%s)",pname,spoolname);
    settitle(name);
}

/*
 *  handle string argument
 *  move to heap
 */
static void 
stringarg(char *arg,char **where, char option)
{
    char *p;

    if (*where) {
	if (option) {
	    fprintf(stderr,"%s: option -%c was \"%s\", respecified as \"%s\"\n",
			    pname,	option, *where, 	     arg);
	} else {
	    fprintf(stderr,"%s: argument was \"%s\", respecified as \"%s\"\n",
			    pname,	      *where,		   arg);
	}
    }
    if (!(p = malloc(strlen(arg)+1))) {
	fprintf(stderr,"%s: out of memory\n",pname);
	unlock_exit(2);
    }
    *where = strcpy(p,arg);
}
